"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("./util");
/**
 * 处理header content-type大小写
 * @param headers
 * @param normalizedName
 */
function normalizeHeaderName(headers, normalizedName) {
    if (!headers)
        return;
    Object.keys(headers).forEach(function (name) {
        if (name !== normalizedName && name.toUpperCase() === normalizedName.toUpperCase()) {
            headers[normalizedName] = headers[name];
            delete headers[name];
        }
    });
}
/**
 * 发送data数据处理content-type
 * @param headers
 * @param data
 */
function processHeaders(headers, data) {
    normalizeHeaderName(headers, 'Content-Type');
    if (util_1.isPlainObject(data)) {
        if (headers && !headers['Content-Type']) {
            headers['Content-Type'] = 'application/json;charset=utf-8';
        }
    }
    return headers;
}
exports.processHeaders = processHeaders;
/**
 * 响应header格式化
 * @param headers
 */
function parseHeaders(headers) {
    var parsed = Object.create(null);
    if (!headers)
        return parsed;
    headers.split('\r\n').forEach(function (line) {
        var _a = line.split(':'), key = _a[0], vals = _a.slice(1);
        key = key.trim().toLowerCase();
        if (!key)
            return;
        parsed[key] = vals.join(':').trim();
    });
    return parsed;
}
exports.parseHeaders = parseHeaders;
/**
 * 合并headers
 * @param headers
 * @param method
 */
function flattenHeaders(headers, method) {
    if (!headers)
        return headers;
    headers = util_1.deepMerge(headers.common, headers[method], headers);
    var methodsToDelete = ['delete', 'get', 'head', 'options', 'post', 'put', 'patch', 'common'];
    methodsToDelete.forEach(function (method) {
        delete headers[method];
    });
    return headers;
}
exports.flattenHeaders = flattenHeaders;
//# sourceMappingURL=headers.js.map